package at.ac.uibk.gitsearch.service.dto;

import at.ac.uibk.gitsearch.domain.ReviewComment;
import at.ac.uibk.gitsearch.domain.enumeration.ReviewStatus;
import java.io.Serializable;
import java.util.Objects;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A DTO for the {@link at.ac.uibk.gitsearch.domain.ReviewRating} entity.
 */
public class ReviewRatingDTO implements Serializable {

    private static final long serialVersionUID = 1L;

    private Long id;

    private ReviewCommentDTO comment;

    private ReviewStatus status;

    private ReviewDTO review;

    private UserDTO user;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public ReviewCommentDTO getComment() {
        return comment;
    }

    public void setComment(ReviewCommentDTO comment) {
        this.comment = comment;
    }

    public ReviewStatus getStatus() {
        return status;
    }

    public void setStatus(ReviewStatus status) {
        this.status = status;
    }

    public ReviewDTO getReview() {
        return review;
    }

    public void setReview(ReviewDTO review) {
        this.review = review;
    }

    public UserDTO getUser() {
        return user;
    }

    public void setUser(UserDTO user) {
        this.user = user;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof ReviewRatingDTO)) {
            return false;
        }

        if (this.id == null) {
            return false;
        }
        ReviewRatingDTO reviewRatingDTO = (ReviewRatingDTO) o;
        return Objects.equals(this.id, reviewRatingDTO.id);
    }

    @Override
    public int hashCode() {
        return Objects.hash(this.id);
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "ReviewRatingDTO{" +
                "id=" + getId() +
                ", comment='" + getComment() + "'" +
                ", status='" + getStatus() + "'" +
                ", review=" + getReview() +
                ", user=" + getUser() +
                "}";
    }

    /**
     * holds a count of the review stati as Map.
     *
     * @author Michael Breu
     *
     */
    public static class ReviewStatisticsDTO implements Serializable {

        private static final Logger LOGGER = LoggerFactory.getLogger(ReviewStatisticsDTO.class);
        /**
         * serialVersionUID
         */
        private static final long serialVersionUID = 7584249169269270589L;

        /*
         * Review In Progress
         * Review improved
         * Review completed
         */
        private int reviewInProgress = 0;
        private int reviewImprovementRequested = 0;
        private int reviewImproved = 0;
        private int reviewCompleted = 0;
        private int reviewRequested = 0;
        private int unknownStats = 0;

        private int reviewsAssignedToReviewer = 0;
        private int reviewImprovedForReviewer = 0;
        private int reviewRejectedByReviewer = 0;
        private int reviewInProgressByReviewer = 0;
        private int reviewImprovementRequestedByReviewer = 0;
        private int reviewCompletedByReviewer = 0;
        private int unkownStatsByReviewer = 0;

        private int badgesRewarded = 0;

        public int getReviewInProgress() {
            return reviewInProgress;
        }

        public void setReviewInProgress(int reviewInProgress) {
            this.reviewInProgress = reviewInProgress;
        }

        public int getReviewImproved() {
            return reviewImproved;
        }

        public void setReviewImproved(int reviewImproved) {
            this.reviewImproved = reviewImproved;
        }

        public int getReviewCompleted() {
            return reviewCompleted;
        }

        public void setReviewCompleted(int reviewCompleted) {
            this.reviewCompleted = reviewCompleted;
        }

        public int getUnknownStats() {
            return unknownStats;
        }

        public void setUnknownStats(int unknownStats) {
            this.unknownStats = unknownStats;
        }

        public void put(ReviewStatus status, int count) {
            if (status == null) {
                reviewRequested = count;
            } else {
                switch (status) {
                    case REVIEW_IN_PROGRESS:
                        setReviewInProgress(count);
                        break;
                    case REVIEW_IMPROVED:
                        setReviewImproved(count);
                        break;
                    case REVIEW_COMPLETED:
                        setReviewCompleted(count);
                        break;
                    case REVIEW_REJECTED:
                        setReviewCompleted(count);
                        break;
                    case IMPROVEMENTS_REQUESTED:
                        setReviewImprovementRequested(count);
                        break;
                    default:
                        LOGGER.error("status {} not yet supported!", status);
                        setUnknownStats(getUnknownStats() + count);
                        break;
                }
            }
        }

        public void putByReviewer(ReviewStatus status, int count) {
            if (status == null) { // this should never happen
                LOGGER.error("Empty status for Reviewer?");
                return;
            }
            switch (status) {
                case REVIEW_IMPROVED:
                    setReviewImprovedForReviewer(count);
                    break;
                case REVIEW_REJECTED:
                    setReviewRejectedByReviewer(count);
                    break;
                case REVIEW_IN_PROGRESS:
                    setReviewInProgressByReviewer(count);
                    break;
                case IMPROVEMENTS_REQUESTED:
                    setReviewImprovementRequestedByReviewer(count);
                    break;
                case REVIEW_COMPLETED:
                    setReviewCompletedByReviewer(count);
                    break;
                default:
                    LOGGER.error("status {} not yet supported!", status);
                    setUnkownStatsByReviewer(getUnkownStatsByReviewer() + count);
                    break;
            }
        }

        public int getReviewInProgressByReviewer() {
            return reviewInProgressByReviewer;
        }

        public void setReviewInProgressByReviewer(int reviewInProgressByReviewer) {
            this.reviewInProgressByReviewer = reviewInProgressByReviewer;
        }

        public int getReviewCompletedByReviewer() {
            return reviewCompletedByReviewer;
        }

        public void setReviewCompletedByReviewer(int reviewCompletedByReviewer) {
            this.reviewCompletedByReviewer = reviewCompletedByReviewer;
        }

        public int getUnkownStatsByReviewer() {
            return unkownStatsByReviewer;
        }

        public void setUnkownStatsByReviewer(int unkownStatsByReviewer) {
            this.unkownStatsByReviewer = unkownStatsByReviewer;
        }

        public int getReviewImprovementRequestedByReviewer() {
            return reviewImprovementRequestedByReviewer;
        }

        public void setReviewImprovementRequestedByReviewer(int reviewImprovementRequestedByReviewer) {
            this.reviewImprovementRequestedByReviewer = reviewImprovementRequestedByReviewer;
        }

        public int getBadgesRewarded() {
            return badgesRewarded;
        }

        public void setBadgesRewarded(int badgedRewarded) {
            this.badgesRewarded = badgedRewarded;
        }

        public int getReviewRequested() {
            return reviewRequested;
        }

        public void setReviewRequested(int reviewRequested) {
            this.reviewRequested = reviewRequested;
        }

        public int getReviewRejectedByReviewer() {
            return reviewRejectedByReviewer;
        }

        public void setReviewRejectedByReviewer(int reviewRejectedByReviewer) {
            this.reviewRejectedByReviewer = reviewRejectedByReviewer;
        }

        public int getReviewImprovedForReviewer() {
            return reviewImprovedForReviewer;
        }

        public void setReviewImprovedForReviewer(int reviewImprovedByReviewer) {
            this.reviewImprovedForReviewer = reviewImprovedByReviewer;
        }

        public int getReviewsAssignedToReviewer() {
            return reviewsAssignedToReviewer;
        }

        public void setReviewsAssignedToReviewer(int reviewsAssignedToReviewer) {
            this.reviewsAssignedToReviewer = reviewsAssignedToReviewer;
        }

        public int getReviewImprovementRequested() {
            return reviewImprovementRequested;
        }

        public void setReviewImprovementRequested(int reviewImprovementRequested) {
            this.reviewImprovementRequested = reviewImprovementRequested;
        }
    }
}
