import { Component } from '@angular/core';
import { SearchInput } from 'app/shared/model/search/search-input.model';
import { SearchService } from 'app/search/service/search-service';
import { SearchResultsDTO } from 'app/shared/model/search/search-results-dto.model';
import { AccountService } from 'app/core/auth/account.service';
import { SearchResultDTO } from 'app/shared/model/search/search-result-dto.model';

@Component({
  selector: 'jhi-search',
  templateUrl: './search.component.html',
  styleUrls: ['./search.component.scss'],
})
export class SearchComponent {
  results: SearchResultDTO[] = [];
  selectedResult?: SearchResultDTO;
  hitCount = 0;
  loading = true;

  private searchInput: SearchInput = new SearchInput();

  constructor(protected searchService: SearchService, public accountService: AccountService) {}

  selectExercise(exercise: SearchResultDTO): void {
    this.selectedResult = exercise;
  }

  updateSearchInput(searchInput: SearchInput): void {
    searchInput.page = 0;
    this.searchInput = searchInput;
    this.results = [];
    this.hitCount = 0;
    this.search();
  }

  private search(): void {
    if (
      this.searchInput.fulltextQuery ||
      this.searchInput.metadata.programmingLanguage ||
      this.searchInput.metadata.keyword ||
      this.searchInput.metadata.author ||
      this.searchInput.metadata.license ||
      this.searchInput.metadata.format ||
      this.searchInput.metadata.naturalLanguage ||
      this.searchInput.metadata.learningResourceType
    ) {
      this.loading = true;
      this.searchService.searchPageDetails(this.searchInput).subscribe({
        next: (data: SearchResultsDTO) => {
          this.results = this.results.concat(data.searchResult);
          ++this.searchInput.page;
          this.loading = false;
        },
        error: () => alert('Search failed'),
      });
    }
  }

  private parseSearchResultsDTO(searchResultsDTO: SearchResultsDTO): SearchResultDTO[] {
    this.hitCount = searchResultsDTO.hitCount;
    // fix string to date conversion: unfortunatelly dates are not converted correctly :-()
    searchResultsDTO.searchResult.forEach(hit => {
      hit.project.last_activity_at = new Date(hit.project.last_activity_at);
      hit.file.indexing_date = new Date(hit.file.indexing_date);
    });
    return searchResultsDTO.searchResult;
  }

  onScroll(): void {
    this.search();
  }
}
